@@ -12,12 +12,20 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tcl.h,v 1.42 1999/04/21 21:50:23 rjohnson Exp $
+ * RCS: @(#) $Id: tcl.h,v 1.63 2000/02/08 10:05:40 hobbs Exp $
  */
 
 #ifndef _TCL
 #define _TCL
 
+/*
+ * For C++ compilers, use extern "C"
+ */
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+    
 /*
  * The following defines are used to indicate the various release levels.
  */
@@ -30,25 +38,30 @@
  * When version numbers change here, must also go into the following files
  * and update the version numbers:
  *
- * library/init.tcl	(only if major.minor changes, not patchlevel)
- * unix/configure.in
- * win/makefile.bc	(only if major.minor changes, not patchlevel)
- * win/makefile.vc	(only if major.minor changes, not patchlevel)
- * win/pkgIndex.tcl	(for tclregNN.dll, not patchlevel)
- * README
- * mac/README
- * win/README.binary
- * win/README		(only if major.minor changes, not patchlevel)
- * unix/README		(only if major.minor changes, not patchlevel)
+ * library/init.tcl	(only if Major.minor changes, not patchlevel) 1 LOC
+ * unix/configure.in	(2 LOC Major, 2 LOC minor, 1 LOC patch)
+ * win/configure.in	(as above)
+ * win/tcl.m4		(not patchlevel)
+ * win/makefile.vc	(not patchlevel) 2 LOC
+ * win/pkgIndex.tcl	(not patchlevel, for tclregNN.dll)
+ * README		(sections 0 and 2)
+ * mac/README		(2 LOC, not patchlevel)
+ * win/README.binary	(sections 0-4)
+ * win/README		(not patchlevel) (sections 0 and 2)
+ * unix/README		(not patchlevel) (part (h))
+ * tests/basic.test	(not patchlevel) (version checks)
+ * tools/tcl.hpj.in	(not patchlevel, for windows installer)
+ * tools/tcl.wse.in	(for windows installer)
+ * tools/tclSplash.bmp	(not patchlevel)
  */
 
 #define TCL_MAJOR_VERSION   8
-#define TCL_MINOR_VERSION   1
+#define TCL_MINOR_VERSION   3
 #define TCL_RELEASE_LEVEL   TCL_FINAL_RELEASE
 #define TCL_RELEASE_SERIAL  0
 
-#define TCL_VERSION	    "8.1"
-#define TCL_PATCH_LEVEL	    "8.1.0"
+#define TCL_VERSION	    "8.3"
+#define TCL_PATCH_LEVEL	    "8.3.0"
 
 /*
  * The following definitions set up the proper options for Windows
@@ -265,6 +278,13 @@
 #   define CONST
 #endif
 
+/*
+ * Make sure EXTERN isn't defined elsewhere
+ */
+#ifdef EXTERN
+#undef EXTERN
+#endif /* EXTERN */
+
 #ifdef __cplusplus
 #   define EXTERN extern "C" TCL_STORAGE_CLASS
 #else
@@ -364,6 +384,63 @@ typedef struct Tcl_TimerToken_ *Tcl_TimerToken;
 typedef struct Tcl_Trace_ *Tcl_Trace;
 typedef struct Tcl_Var_ *Tcl_Var;
 
+/*
+ * Flag values passed to Tcl_GetRegExpFromObj.
+ */
+
+#define	TCL_REG_BASIC		000000	/* BREs (convenience) */
+#define	TCL_REG_EXTENDED	000001	/* EREs */
+#define	TCL_REG_ADVF		000002	/* advanced features in EREs */
+#define	TCL_REG_ADVANCED	000003	/* AREs (which are also EREs) */
+#define	TCL_REG_QUOTE		000004	/* no special characters, none */
+#define	TCL_REG_NOCASE		000010	/* ignore case */
+#define	TCL_REG_NOSUB		000020	/* don't care about subexpressions */
+#define	TCL_REG_EXPANDED	000040	/* expanded format, white space &
+					 * comments */
+#define	TCL_REG_NLSTOP		000100  /* \n doesn't match . or [^ ] */
+#define	TCL_REG_NLANCH		000200  /* ^ matches after \n, $ before */
+#define	TCL_REG_NEWLINE		000300  /* newlines are line terminators */
+#define	TCL_REG_CANMATCH	001000  /* report details on partial/limited
+					 * matches */
+
+/*
+ * The following flag is experimental and only intended for use by Expect.  It
+ * will probably go away in a later release.
+ */
+
+#define TCL_REG_BOSONLY		002000	/* prepend \A to pattern so it only
+					 * matches at the beginning of the
+					 * string. */
+
+/*
+ * Flags values passed to Tcl_RegExpExecObj.
+ */
+
+#define	TCL_REG_NOTBOL	0001	/* Beginning of string does not match ^.  */
+#define	TCL_REG_NOTEOL	0002	/* End of string does not match $. */
+
+/*
+ * Structures filled in by Tcl_RegExpInfo.  Note that all offset values are
+ * relative to the start of the match string, not the beginning of the
+ * entire string.
+ */
+
+typedef struct Tcl_RegExpIndices {
+    long start;		/* character offset of first character in match */
+    long end;		/* character offset of first character after the
+			 * match. */
+} Tcl_RegExpIndices;
+
+typedef struct Tcl_RegExpInfo {
+    int nsubs;			/* number of subexpressions in the
+				 * compiled expression */
+    Tcl_RegExpIndices *matches;	/* array of nsubs match offset
+				 * pairs */
+    long extendStart;		/* The offset at which a subsequent
+				 * match might begin. */
+    long reserved;		/* Reserved for later use. */
+} Tcl_RegExpInfo;
+
 /*
  * Picky compilers complain if this typdef doesn't appear before the
  * struct's reference in tclDecls.h.
@@ -472,6 +549,9 @@ typedef int (Tcl_SetFromAnyProc) _ANSI_ARGS_((Tcl_Interp *interp,
 typedef void (Tcl_UpdateStringProc) _ANSI_ARGS_((struct Tcl_Obj *objPtr));
 typedef char *(Tcl_VarTraceProc) _ANSI_ARGS_((ClientData clientData,
 	Tcl_Interp *interp, char *part1, char *part2, int flags));
+typedef void (Tcl_CreateFileHandlerProc) _ANSI_ARGS_((int fd, int mask,
+	Tcl_FileProc *proc, ClientData clientData));
+typedef void (Tcl_DeleteFileHandlerProc) _ANSI_ARGS_((int fd));
 
 /*
  * The following structure represents a type of object, which is a
@@ -545,9 +625,9 @@ typedef struct Tcl_Obj {
  * expression that is expensive to compute or has side effects.
  */
 
-EXTERN void		Tcl_IncrRefCount _ANSI_ARGS_((Tcl_Obj *objPtr));
-EXTERN void		Tcl_DecrRefCount _ANSI_ARGS_((Tcl_Obj *objPtr));
-EXTERN int		Tcl_IsShared _ANSI_ARGS_((Tcl_Obj *objPtr));
+void		Tcl_IncrRefCount _ANSI_ARGS_((Tcl_Obj *objPtr));
+void		Tcl_DecrRefCount _ANSI_ARGS_((Tcl_Obj *objPtr));
+int		Tcl_IsShared _ANSI_ARGS_((Tcl_Obj *objPtr));
 
 #ifdef TCL_MEM_DEBUG
 #   define Tcl_IncrRefCount(objPtr) \
@@ -610,7 +690,7 @@ typedef struct Tcl_SavedResult {
 /*
  * The following definitions support Tcl's namespace facility.
  * Note: the first five fields must match exactly the fields in a
- * Namespace structure (see tcl.h). 
+ * Namespace structure (see tclInt.h). 
  */
 
 typedef struct Tcl_Namespace {
@@ -985,6 +1065,9 @@ typedef struct Tcl_Time {
     long usec;			/* Microseconds. */
 } Tcl_Time;
 
+typedef void (Tcl_SetTimerProc) _ANSI_ARGS_((Tcl_Time *timePtr));
+typedef int (Tcl_WaitForEventProc) _ANSI_ARGS_((Tcl_Time *timePtr));
+
 /*
  * Bits to pass to Tcl_CreateFileHandler and Tcl_CreateChannelHandler
  * to indicate what sorts of events are of interest:
@@ -1063,22 +1146,15 @@ typedef int	(Tcl_DriverGetHandleProc) _ANSI_ARGS_((
 #else /* !TCL_MEM_DEBUG */
 
 /*
- * If USE_TCLALLOC is true, then we need to call Tcl_Alloc instead of
- * the native malloc/free.  The only time USE_TCLALLOC should not be
- * true is when compiling the Tcl/Tk libraries on Unix systems.  In this
- * case we can safely call the native malloc/free directly as a performance
- * optimization.
+ * If we are not using the debugging allocator, we should call the 
+ * Tcl_Alloc, et al. routines in order to guarantee that every module
+ * is using the same memory allocator both inside and outside of the
+ * Tcl library.
  */
 
-#   if USE_TCLALLOC
-#	define ckalloc(x) Tcl_Alloc(x)
-#	define ckfree(x) Tcl_Free(x)
-#	define ckrealloc(x,y) Tcl_Realloc(x,y)
-#   else
-#	define ckalloc(x) malloc(x)
-#	define ckfree(x)  free(x)
-#	define ckrealloc(x,y) realloc(x,y)
-#   endif
+#   define ckalloc(x) Tcl_Alloc(x)
+#   define ckfree(x) Tcl_Free(x)
+#   define ckrealloc(x,y) Tcl_Realloc(x,y)
 #   define Tcl_InitMemory(x)
 #   define Tcl_DumpActiveMemory(x)
 #   define Tcl_ValidateAllMemory(x,y)
@@ -1156,6 +1232,18 @@ typedef enum Tcl_PathType {
     TCL_PATH_VOLUME_RELATIVE
 } Tcl_PathType;
 
+/*
+ * The following structure represents the Notifier functions that
+ * you can override with the Tcl_SetNotifier call.
+ */
+
+typedef struct Tcl_NotifierProcs {
+    Tcl_SetTimerProc *setTimerProc;
+    Tcl_WaitForEventProc *waitForEventProc;
+    Tcl_CreateFileHandlerProc *createFileHandlerProc;
+    Tcl_DeleteFileHandlerProc *deleteFileHandlerProc;
+} Tcl_NotifierProcs;
+
 /*
  * The following structure represents a user-defined encoding.  It collects
  * together all the functions that are used by the specific encoding.
@@ -1543,4 +1631,12 @@ EXTERN int		Tcl_AppInit _ANSI_ARGS_((Tcl_Interp *interp));
 #undef TCL_STORAGE_CLASS
 #define TCL_STORAGE_CLASS DLLIMPORT
 
+/*
+ * end block for C++
+ */
+    
+#ifdef __cplusplus
+}
+#endif
+    
 #endif /* _TCL */
