@@ -3,7 +3,7 @@
 # Default system startup file for Tcl-based applications.  Defines
 # "unknown" procedure and auto-load facilities.
 #
-# RCS: @(#) $Id: init.tcl,v 1.32.4.1 1999/10/30 11:06:17 hobbs Exp $
+# RCS: @(#) $Id: init.tcl,v 1.39 2000/02/01 19:26:08 ericm Exp $
 #
 # Copyright (c) 1991-1993 The Regents of the University of California.
 # Copyright (c) 1994-1996 Sun Microsystems, Inc.
@@ -16,7 +16,7 @@
 if {[info commands package] == ""} {
     error "version mismatch: library\nscripts expect Tcl version 7.5b1 or later but the loaded version is\nonly [info patchlevel]"
 }
-package require -exact Tcl 8.2
+package require -exact Tcl 8.3
 
 # Compute the auto path to use in this interpreter.
 # The values on the path come from several locations:
@@ -52,8 +52,8 @@ if {[string compare [info library] {}]} {
 	}
     }
 }
-set __dir [file join [file dirname \
-	[file dirname [info nameofexecutable]]] lib]
+set __dir [file join [file dirname [file dirname \
+	[info nameofexecutable]]] lib]
 if {[lsearch -exact $auto_path $__dir] < 0} {
     lappend auto_path $__dir
 }
@@ -70,7 +70,7 @@ if {[info exists __dir]} {
   
 # Windows specific end of initialization
 
-if {(![interp issafe]) && ($tcl_platform(platform) == "windows")} {
+if {(![interp issafe]) && [string equal $tcl_platform(platform) "windows"]} {
     namespace eval tcl {
 	proc envTraceProc {lo n1 n2 op} {
 	    set x $::env($n2)
@@ -80,7 +80,7 @@ if {(![interp issafe]) && ($tcl_platform(platform) == "windows")} {
     }
     foreach p [array names env] {
 	set u [string toupper $p]
-	if {$u != $p} {
+	if {[string compare $u $p]} {
 	    switch -- $u {
 		COMSPEC -
 		PATH {
@@ -100,7 +100,7 @@ if {(![interp issafe]) && ($tcl_platform(platform) == "windows")} {
 	unset u
     }
     if {![info exists env(COMSPEC)]} {
-	if {$tcl_platform(os) == {Windows NT}} {
+	if {[string equal $tcl_platform(os) "Windows NT"]} {
 	    set env(COMSPEC) cmd.exe
 	} else {
 	    set env(COMSPEC) command.com
@@ -114,7 +114,7 @@ package unknown tclPkgUnknown
 
 # Conditionalize for presence of exec.
 
-if {[info commands exec] == ""} {
+if {[llength [info commands exec]] == 0} {
 
     # Some machines, such as the Macintosh, do not have exec. Also, on all
     # platforms, safe interpreters do not have exec.
@@ -127,7 +127,7 @@ set errorInfo ""
 # Define a log command (which can be overwitten to log errors
 # differently, specially when stderr is not available)
 
-if {[info commands tclLog] == ""} {
+if {[llength [info commands tclLog]] == 0} {
     proc tclLog {string} {
 	catch {puts stderr $string}
     }
@@ -191,8 +191,8 @@ proc unknown args {
 	set ret [catch {auto_load $name [uplevel 1 {namespace current}]} msg]
 	unset unknown_pending($name);
 	if {$ret != 0} {
-	    return -code $ret -errorcode $errorCode \
-		"error while autoloading \"$name\": $msg"
+	    append errorInfo "\n    (autoloading \"$name\")"
+	    return -code $ret -errorcode $errorCode -errorinfo $errorInfo $msg
 	}
 	if {![array size unknown_pending]} {
 	    unset unknown_pending
@@ -217,7 +217,7 @@ proc unknown args {
 	}
     }
 
-    if {([info level] == 1) && ([info script] == "") \
+    if {([info level] == 1) && [string equal [info script] ""] \
 	    && [info exists tcl_interactive] && $tcl_interactive} {
 	if {![info exists auto_noexec]} {
 	    set new [auto_execok $name]
@@ -225,7 +225,7 @@ proc unknown args {
 		set errorCode $savedErrorCode
 		set errorInfo $savedErrorInfo
 		set redir ""
-		if {[info commands console] == ""} {
+		if {[string equal [info commands console] ""]} {
 		    set redir ">&@stdout <@stdin"
 		}
 		return [uplevel exec $redir $new [lrange $args 1 end]]
@@ -233,7 +233,7 @@ proc unknown args {
 	}
 	set errorCode $savedErrorCode
 	set errorInfo $savedErrorInfo
-	if {$name == "!!"} {
+	if {[string equal $name "!!"]} {
 	    set newcmd [history event]
 	} elseif {[regexp {^!(.+)$} $name dummy event]} {
 	    set newcmd [history event $event]
@@ -248,7 +248,7 @@ proc unknown args {
 	}
 
 	set ret [catch {set cmds [info commands $name*]} msg]
-	if {[string compare $name "::"] == 0} {
+	if {[string equal $name "::"]} {
 	    set name ""
 	}
 	if {$ret != 0} {
@@ -258,8 +258,8 @@ proc unknown args {
 	if {[llength $cmds] == 1} {
 	    return [uplevel [lreplace $args 0 0 $cmds]]
 	}
-	if {[llength $cmds] != 0} {
-	    if {$name == ""} {
+	if {[llength $cmds]} {
+	    if {[string equal $name ""]} {
 		return -code error "empty command name \"\""
 	    } else {
 		return -code error \
@@ -305,11 +305,17 @@ proc auto_load {cmd {namespace {}}} {
     if {![auto_load_index]} {
 	return 0
     }
-
     foreach name $nameList {
 	if {[info exists auto_index($name)]} {
 	    uplevel #0 $auto_index($name)
-	    if {[info commands $name] != ""} {
+	    # There's a couple of ways to look for a command of a given
+	    # name.  One is to use
+	    #    info commands $name
+	    # Unfortunately, if the name has glob-magic chars in it like *
+	    # or [], it may not match.  For our purposes here, a better
+	    # route is to use 
+	    #    namespace which -command $name
+	    if { ![string equal [namespace which -command $name] ""] } {
 		return 1
 	    }
 	}
@@ -329,10 +335,9 @@ proc auto_load {cmd {namespace {}}} {
 proc auto_load_index {} {
     global auto_index auto_oldpath auto_path errorInfo errorCode
 
-    if {[info exists auto_oldpath]} {
-	if {$auto_oldpath == $auto_path} {
-	    return 0
-	}
+    if {[info exists auto_oldpath] && \
+	    [string equal $auto_oldpath $auto_path]} {
+	return 0
     }
     set auto_oldpath $auto_path
 
@@ -350,25 +355,24 @@ proc auto_load_index {} {
 	} else {
 	    set error [catch {
 		set id [gets $f]
-		if {$id == "# Tcl autoload index file, version 2.0"} {
+		if {[string equal $id \
+			"# Tcl autoload index file, version 2.0"]} {
 		    eval [read $f]
-		} elseif {$id == \
-		    "# Tcl autoload index file: each line identifies a Tcl"} {
+		} elseif {[string equal $id "# Tcl autoload index file: each line identifies a Tcl"]} {
 		    while {[gets $f line] >= 0} {
-			if {([string index $line 0] == "#")
+			if {[string equal [string index $line 0] "#"] \
 				|| ([llength $line] != 2)} {
 			    continue
 			}
 			set name [lindex $line 0]
 			set auto_index($name) \
-			    "source [file join $dir [lindex $line 1]]"
+				"source [file join $dir [lindex $line 1]]"
 		    }
 		} else {
-		    error \
-		      "[file join $dir tclIndex] isn't a proper Tcl index file"
+		    error "[file join $dir tclIndex] isn't a proper Tcl index file"
 		}
 	    } msg]
-	    if {$f != ""} {
+	    if {[string compare $f ""]} {
 		close $f
 	    }
 	    if {$error} {
@@ -421,21 +425,19 @@ proc auto_qualify {cmd namespace} {
     # (if the current namespace is not the global one)
 
     if {$n == 0} {
-	if {[string compare $namespace ::] == 0} {
+	if {[string equal $namespace ::]} {
 	    # ( nocolons , :: ) -> nocolons
 	    return [list $cmd]
 	} else {
 	    # ( nocolons , ::sub ) -> ::sub::nocolons nocolons
 	    return [list ${namespace}::$cmd $cmd]
 	}
+    } elseif {[string equal $namespace ::]} {
+	#  ( foo::bar , :: ) -> ::foo::bar
+	return [list ::$cmd]
     } else {
-	if {[string compare $namespace ::] == 0} {
-	    #  ( foo::bar , :: ) -> ::foo::bar
-	    return [list ::$cmd]
-	} else {
-	    # ( foo::bar , ::sub ) -> ::sub::foo::bar ::foo::bar
-	    return [list ${namespace}::$cmd ::$cmd]
-	}
+	# ( foo::bar , ::sub ) -> ::sub::foo::bar ::foo::bar
+	return [list ${namespace}::$cmd ::$cmd]
     }
 }
 
@@ -460,7 +462,8 @@ proc auto_import {pattern} {
 
     foreach pattern $patternList {
         foreach name [array names auto_index] {
-            if {[string match $pattern $name] && "" == [info commands $name]} {
+            if {[string match $pattern $name] && \
+		    [string equal "" [info commands $name]]} {
                 uplevel #0 $auto_index($name)
             }
         }
@@ -494,8 +497,14 @@ proc auto_execok name {
     }
     set auto_execs($name) ""
 
-    if {[lsearch -exact {cls copy date del erase dir echo mkdir md rename 
-	    ren rmdir rd time type ver vol} $name] != -1} {
+    set shellBuiltins [list cls copy date del erase dir echo mkdir \
+	    md rename ren rmdir rd time type ver vol]
+    if {[string equal $tcl_platform(os) "Windows NT"]} {
+	# NT includes the 'start' built-in
+	lappend shellBuiltins "start"
+    }
+
+    if {[lsearch -exact $shellBuiltins $name] != -1} {
 	return [set auto_execs($name) [list $env(COMSPEC) /c $name]]
     }
 
@@ -514,7 +523,7 @@ proc auto_execok name {
 	set windir $env(WINDIR) 
     }
     if {[info exists windir]} {
-	if {$tcl_platform(os) == "Windows NT"} {
+	if {[string equal $tcl_platform(os) "Windows NT"]} {
 	    append path "$windir/system32;"
 	}
 	append path "$windir/system;$windir;"
@@ -557,7 +566,7 @@ proc auto_execok name {
 	return $auto_execs($name)
     }
     foreach dir [split $env(PATH) :] {
-	if {$dir == ""} {
+	if {[string equal $dir ""]} {
 	    set dir .
 	}
 	set file [file join $dir $name]
