@@ -5,18 +5,19 @@
  *  implementing Macintosh specific portions of the Tcl port.
  *
  * Copyright (c) 1993-1994 Lockheed Missle & Space Company, AI Center
- * Copyright (c) 1995-1996 Sun Microsystems, Inc.
+ * Copyright (c) 1995-1997 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tclMacUtil.c 1.50 96/10/04 05:07:15
+ * RCS: @(#) $Id: tclMacUtil.c,v 1.3 1998/11/10 06:49:55 jingham Exp $
  */
 
 #include "tcl.h"
 #include "tclInt.h"
 #include "tclMacInt.h"
-#include <math.h>
+#include "tclMath.h"
+#include "tclMacPort.h"
 
 #include <Aliases.h>
 #include <Errors.h>
@@ -36,10 +37,12 @@
 /*
  *----------------------------------------------------------------------
  *
- * hypot --
+ * hypotd --
  *
  *	The standard math function hypot is not supported by Think C.
- *	It is included here so everything works.
+ *	It is included here so everything works. It is supported by
+ *	CodeWarrior Pro 1, but the 68K version does not support doubles.
+ *	So we hack it in.
  *
  * Results:
  *	Result of computation.
@@ -50,11 +53,11 @@
  *----------------------------------------------------------------------
  */
  
-#if defined(THINK_C) || GENERATING68K
-double hypot(double x, double y);
+#if defined(THINK_C) || defined(__MWERKS__)
+double hypotd(double x, double y);
 
 double
-hypot(
+hypotd(
     double x,		/* X value */
     double y)		/* Y value */
 {
@@ -198,7 +201,7 @@ FSpFindFolder(
 int
 FSpLocationFromPath(
     int length,			/* Length of path. */
-    char *path,			/* The path to convert. */
+    CONST char *path,		/* The path to convert. */
     FSSpecPtr fileSpecPtr)	/* On return the spec for the path. */
 {
     Str255 fileName;
