@@ -10,7 +10,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclUnixPipe.c,v 1.3.6.1 1999/10/30 11:09:45 hobbs Exp $
+ * RCS: @(#) $Id: tclUnixPipe.c,v 1.7 1999/12/12 02:27:20 hobbs Exp $
  */
 
 #include "tclInt.h"
@@ -353,7 +353,7 @@ TclpCreateProcess(interp, argc, argv, inputFile, outputFile, errorFile,
     }
 
     /*
-     * We need to allocate and convert this before the (v)fork
+     * We need to allocate and convert this before the fork
      * so it is properly deallocated later
      */
     dsArray = (Tcl_DString *) ckalloc(argc * sizeof(Tcl_DString));
@@ -364,7 +364,7 @@ TclpCreateProcess(interp, argc, argv, inputFile, outputFile, errorFile,
     }
 
     joinThisError = (errorFile == outputFile);
-    pid = vfork();
+    pid = fork();
     if (pid == 0) {
 	fd = GetFd(errPipeOut);
 
@@ -776,7 +776,6 @@ PipeBlockModeProc(instanceData, mode)
         if (fcntl(fd, F_SETFL, curStatus) < 0) {
             return errno;
         }
-        curStatus = fcntl(fd, F_GETFL);
     }
     if (psPtr->outFile) {
         fd = GetFd(psPtr->outFile);
@@ -816,7 +815,9 @@ PipeBlockModeProc(instanceData, mode)
         }
     }
 #endif	/* USE_FIONBIO */
-    
+
+    psPtr->isNonBlocking = (mode == TCL_MODE_NONBLOCKING);
+
     return 0;
 }
 
